"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const path_1 = __importDefault(require("path"));
const inquirer_1 = __importDefault(require("inquirer"));
const ejs_1 = __importDefault(require("ejs"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const utils_1 = require("./utils");
const request_1 = require("./request");
const resolveConfig_1 = require("./resolveConfig");
const __config__ = (0, resolveConfig_1.resolveConfig)();
const formatType = (list, type) => {
    // 基础类型
    const typeMapping = {
        File: "string",
        Variable: "string",
        String: "string",
        Number: "number",
        Boolean: "boolean",
        Guid: "string",
        Int: "number",
        long: "number",
        int: "number"
    };
    return list.map((v) => {
        let res = {};
        if (typeMapping[v.typeName]) {
            res = {
                ...v,
                typeName: typeMapping[v.typeName],
            };
        }
        else {
            const find = type.find((t) => t.id === v.type);
            if (find) {
                const isEnum = find.name.includes("Enum");
                let enumStr = find.params
                    .map((vv) => `\'${vv.name}\'`)
                    .join(" | ");
                res = {
                    ...v,
                    typeName: isEnum ? enumStr : formatType(find.params, type),
                };
            }
        }
        return res;
    });
};
// 递归
const fn = (list) => list
    .map((v) => `${v.name}${v.required === 0 ? '?' : ''}:${Array.isArray(v.typeName) ? `{${fn(v.typeName)}}[]` : v.typeName};`)
    .join("\n");
const generateFile = (interfaces) => {
    const pathArr = interfaces[0].path.split("/");
    const filename = (0, utils_1.toLowerCamelCase)(`${pathArr[pathArr.length - 2]}.ts`);
    const list = interfaces.map((v) => {
        const pathArr = v.path.split("/");
        const name = pathArr[pathArr.length - 1];
        return {
            comment: v.name,
            nameLower: (0, utils_1.toLowerCamelCase)(name),
            nameUpper: (0, utils_1.toUpperCamelCase)(name),
            type: v.method.toLowerCase(),
            url: v.path,
        };
    });
    const ejsFolder = __config__.ejs || path_1.default.join(__dirname, "..", "template");
    const ejsUrl = path_1.default.join(ejsFolder, "template.ejs");
    const templateStr = fs_extra_1.default.readFileSync(ejsUrl, {
        encoding: "utf-8",
    });
    const file = ejs_1.default.render(templateStr, { list });
    const dir = __config__.distDir;
    const url = `${dir}/${filename}`;
    try {
        fs_extra_1.default.accessSync(dir, fs_extra_1.default.constants.F_OK);
    }
    catch (error) {
        fs_extra_1.default.mkdirSync(dir, { recursive: true });
    }
    finally {
        fs_extra_1.default.writeFileSync(url, file);
        console.log(`写入成功`);
    }
};
const generateType = async (interfaces, type, distType) => {
    const pathArr = interfaces[0].path.split("/");
    const suffix = distType === "inner" ? "" : ".d";
    const filename = (0, utils_1.toLowerCamelCase)(`${pathArr[pathArr.length - 2]}${suffix}.ts`);
    const list = interfaces.map((v) => {
        const pathArr = v.path.split("/");
        const name = pathArr[pathArr.length - 1];
        return {
            comment: v.name,
            nameLower: (0, utils_1.toLowerCamelCase)(name),
            nameUpper: (0, utils_1.toUpperCamelCase)(name),
            type: v.method.toLowerCase(),
            url: v.path,
            inputs: formatType(v.params.inputs, type) || [],
            outputs: formatType(v.params.outputs, type) || [],
        };
    });
    const ejsFolder = __config__.ejs || path_1.default.join(__dirname, "..", "template");
    const ejsUrl = path_1.default.join(ejsFolder, distType === "inner" ? "templateAll.ejs" : "templateType.ejs");
    const templateStr = fs_extra_1.default.readFileSync(ejsUrl, { encoding: "utf-8" });
    const file = ejs_1.default.render(templateStr, { list, fn });
    const dir = __config__.distDir;
    const url = `${dir}/${filename}`;
    try {
        fs_extra_1.default.accessSync(dir, fs_extra_1.default.constants.F_OK);
    }
    catch (error) {
        fs_extra_1.default.mkdirSync(dir, { recursive: true });
    }
    finally {
        fs_extra_1.default.writeFileSync(url, file);
        console.log(`写入成功`);
    }
};
const generate = async (toolKey) => {
    const { groups, interfaces, datatypes } = await (0, request_1.getProjectres)(toolKey);
    if (!__config__.gid) {
        const answer = await inquirer_1.default.prompt([
            {
                type: "list",
                message: "请选择分组: ",
                choices: groups.map((v) => v.name),
                name: "group",
                default: "",
            },
        ]);
        const currentGroup = groups.find((v) => v.name === answer.group);
        __config__.gid = currentGroup.id;
    }
    // 根据gid生成
    const interfaceList = interfaces.filter((v) => v.group.id === Number(__config__.gid));
    // const typeList = datatypes.filter(v => v.group.id === Number(__config__.gid))
    if (__config__.distType === "outer") {
        generateFile(interfaceList);
    }
    generateType(interfaceList, datatypes, __config__.distType);
};
exports.generate = generate;
//# sourceMappingURL=generate.js.map